/* eslint-disable no-console, spaced-comment */

// create an empty modbus client
//var ModbusRTU = require("modbus-serial");
var ModbusRTU = require("../index");
var client = new ModbusRTU();
var moment = require('moment')

var bsc_error_codes = [{
        code: 'BSC_LGCHEM_01000001',
        level: 0x03,
        desc: "Over Charge Current Warning"
    },
    {
        code: 'BSC_LGCHEM_01000002',
        level: 0x03,
        desc: "Over Discharge Current Warning"
    },
    {
        code: 'BSC_LGCHEM_01000004',
        level: 0x03,
        desc: "Over Charge Power Limit Warning"
    },
    {
        code: 'BSC_LGCHEM_01000008',
        level: 0x03,
        desc: "Over Discharge Pover Limit Warning"
    },
    {
        code: 'BSC_LGCHEM_01000010',
        level: 0x03,
        desc: "Some racks are in fault state"
    },
    {
        code: 'BSC_LGCHEM_01000020',
        level: 0x03,
        desc: "All Racks Warning"
    },
    {
        code: 'BSC_LGCHEM_01000040',
        level: 0x03,
        desc: "Over Average Temperature Warning"
    },
    {
        code: 'BSC_LGCHEM_01000080',
        level: 0x03,
        desc: "Over Container Temperature Warning"
    },
    null, null, null, null, null, null, null,
    {
        code: 'BSC_LGCHEM_00000001',
        level: 0x00,
        desc: "Over Charge Current Fault"
    },
    {
        code: 'BSC_LGCHEM_00000002',
        level: 0x00,
        desc: "Over Discharge Current Fault"
    },
    {
        code: 'BSC_LGCHEM_00000004',
        level: 0x00,
        desc: "Over Charge Power Limit Fault"
    },
    {
        code: 'BSC_LGCHEM_00000008',
        level: 0x00,
        desc: "Over Discharge Power Limit Fault"
    },
    {
        code: 'BSC_LGCHEM_00000010',
        level: 0x00,
        desc: "Over Offline Rack Count Fault"
    },
    {
        code: 'BSC_LGCHEM_00000020',
        level: 0x00,
        desc: "Initialization Fault"
    },
    {
        code: 'BSC_LGCHEM_00000040',
        level: 0x00,
        desc: "All Racks Fault"
    },
    {
        code: 'BSC_LGCHEM_00000080',
        level: 0x00,
        desc: "Over Average Temperature Fault"
    },
    {
        code: 'BSC_LGCHEM_00000100',
        level: 0x00,
        desc: "Over Container Temperature Fault"
    },
    null, null, null, null, null, null, null
]
var bsc_protection_status_length = 16

var bsc_rack_error_codes = [{
        code: "BSC_LGCHEM_40000001",
        level: 0x03,
        desc: "RACK : Over Charge Current Warning"
    },
    {
        code: "BSC_LGCHEM_40000002",
        level: 0x03,
        desc: "RACK : Over Discharge Current Warning"
    },

    {
        code: "BSC_LGCHEM_40000004",
        level: 0x03,
        desc: "RACK : Over Charge Power Limit Warning"
    },
    {
        code: "BSC_LGCHEM_40000008",
        level: 0x03,
        desc: "RACK : Over Discharge Pover Limit Warning"
    },
    {
        code: "BSC_LGCHEM_40000010",
        level: 0x03,
        desc: "RACK : Over Cell Voltage Warning"
    },
    {
        code: "BSC_LGCHEM_40000020",
        level: 0x03,
        desc: "RACK : Under Cell Voltage Warning"
    },
    {
        code: "BSC_LGCHEM_40000040",
        level: 0x03,
        desc: "RACK : Over Cell Voltage Difference Warning"
    },
    {
        code: "BSC_LGCHEM_40000080",
        level: 0x03,
        desc: "RACK : Over Module Temperature Warning"
    },
    {
        code: "BSC_LGCHEM_40000100",
        level: 0x03,
        desc: "RACK : Under Module Temperature Warning"
    },
    {
        code: "BSC_LGCHEM_40000200",
        level: 0x03,
        desc: "RACK : Over Module Temperature Difference Warning"
    },
    {
        code: "BSC_LGCHEM_40000400",
        level: 0x03,
        desc: "RACK : Over Module Temperature Warning (Long Term)"
    },
    null, null, null, null, null,
    {
        code: "BSC_LGCHEM_10000001",
        level: 0x01,
        desc: "RACK : Over Charge Current Fault"
    },
    {
        code: "BSC_LGCHEM_10000002",
        level: 0x01,
        desc: "RACK : Over Discharge Current Fault"
    },

    {
        code: "BSC_LGCHEM_10000004",
        level: 0x01,
        desc: "RACK : Over Charge Power Limit Fault"
    }, {
        code: "BSC_LGCHEM_10000008",
        level: 0x01,
        desc: "RACK : Over Discharge Pover Limit Fault"
    }, {
        code: "BSC_LGCHEM_10000010",
        level: 0x01,
        desc: "RACK : Over Cell Voltage Fauilt"
    }, {
        code: "BSC_LGCHEM_10000020",
        level: 0x01,
        desc: "RACK : Under Cell Voltage Fault"
    }, {
        code: "BSC_LGCHEM_10000040",
        level: 0x01,
        desc: "RACK : Over Cell Voltage Difference Fault"
    }, {
        code: "BSC_LGCHEM_10000080",
        level: 0x01,
        desc: "RACK : Over Module Temperature Fault"
    }, {
        code: "BSC_LGCHEM_10000100",
        level: 0x01,
        desc: "RACK : Under Module Temperature Fault"
    }, {
        code: "BSC_LGCHEM_10000200",
        level: 0x01,
        desc: "RACK : Over Module Temperature Difference Fault"
    }, {
        code: "BSC_LGCHEM_10000400",
        level: 0x01,
        desc: "RACK : Over Module Temperature Fault (Long Term)"
    },
    null, null, null, null, null, {
        code: "BSC_LGCHEM_20000001",
        level: 0x01,
        desc: "RACK : Comm. Error with the BSC"
    }, {
        code: "BSC_LGCHEM_20000002",
        level: 0x01,
        desc: "RACK : Comm. Error with MBMSs"
    },

    {
        code: "BSC_LGCHEM_20000004",
        level: 0x01,
        desc: "RACK : Current Sensing Error"
    }, {
        code: "BSC_LGCHEM_20000008",
        level: 0x01,
        desc: "RACK : Temperature Sensing Error"
    }, {
        code: "BSC_LGCHEM_20000010",
        level: 0x01,
        desc: "RACK : Rack Unbalancing Error"
    }, {
        code: "BSC_LGCHEM_20000020",
        level: 0x01,
        desc: "RACK : Disconnect Switch(or Circuit breaker) Feedback Error"
    }, {
        code: "BSC_LGCHEM_20000040",
        level: 0x01,
        desc: "RACK : Main Contactor Feedback Error"
    }, {
        code: "BSC_LGCHEM_20000080",
        level: 0x01,
        desc: "RACK : Fan Feedback Error"
    }, {
        code: "BSC_LGCHEM_20000100",
        level: 0x01,
        desc: "RACK : UMBMS OBD Error"
    }, {
        code: "BSC_LGCHEM_20000200",
        level: 0x01,
        desc: "RACK : Rack Connection Fault"
    }, {
        code: "BSC_LGCHEM_20000400",
        level: 0x01,
        desc: "RACK : Initialization Fault"
    }, {
        code: "BSC_LGCHEM_20000800",
        level: 0x01,
        desc: "RACK : Conntactor Open Fault"
    },
    null, null, null, null
]

// open connection to a serial port
//client.connectRTU("/dev/ttyUSB0", {baudRate: 9600})
client.connectTCP("sinan-ecs.nbplus.kr", {
        port: 13502
    })
    .then(setClient)
    .then(function() {
        console.log("Connected");
    })
    .catch(function(e) {
        console.log(e.message);
    });

function setClient() {
    // set the client's unit id
    // set a timout for requests default is null (no timeout)
    client.setID(1);
    client.setTimeout(1000);

    // run program
    run();
}

async function run() {
    client.setID(1)
    try {
        var response = await client.readInputRegisters(1, 50)
        if (response) {
            console.log('Received data =', response)

            var monitoringData = {}

            var buff_pos = 0
            monitoringData.op_status = response.data[1]
            monitoringData.info = response.data[2]
            // set bsc_charge_status
            monitoringData.charge_status = (response.data[2] & 0x000C) >> 2
            monitoringData.has_warning = ((response.data[2] & 0x0001) === 0x01)
            monitoringData.has_fault = (((response.data[2] & 0x0002) >> 1) === 0x01)

            monitoringData.ntotal_racks = response.data[3]
            monitoringData.nonline_racks = response.data[4]
            monitoringData.nwarning_racks = response.data[5]
            monitoringData.nfault_racks = response.data[6]
            monitoringData.ncb_closed_racks = response.data[7]
            monitoringData.nfanon_racks = response.data[8]
            monitoringData.ncell_balancing_racks = response.data[9]
            monitoringData.warning_codes = response.data[10]
            monitoringData.fault_codes = response.data[11]
            // set error event and protection status
            var errbyte
            var byte_pos = buff_pos + 18 // from response.data[9] to response.data[10]
            var op = 0x01

            var date = moment().format('YYYY-MM-DD HH:mm:ss')
            monitoringData.protection_status = {}
            for (var i = 0; i < bsc_error_codes.length; i++) {
                if (isNullObject(bsc_error_codes[i])) {
                    continue
                }
                var bit_pos = i % 8
                if (bit_pos == 0) {
                    errbyte = response.buffer.readInt8(byte_pos + (i / 8))
                }

                var isError = (((errbyte >> bit_pos) & op) == op)
                // PCS_DESTIN_00000001: {
                //     desc: 'PCS Start Failure',
                //     err_yn: false
                // },
                if (i >= bsc_protection_status_length) {
                    monitoringData.protection_status[bsc_error_codes[i].code] = {
                        desc: bsc_error_codes[i].desc,
                        err_yn: isError
                    }
                }
                if (!isError) {
                    continue
                }
                // ECS_00000002: {
                //     level: 0x00,
                //     desc: 'PCS 통신 장애',
                //     date: date
                // }
                error.events[bsc_error_codes[i].code] = {
                    level: bsc_error_codes[i].level,
                    desc: bsc_error_codes[i].desc,
                    num_rack: 0,
                    date: date
                }
            }

            monitoringData.soc = +(response.data[12] / 10.0).toFixed(1)
            // set soc_kwh
            monitoringData.soc_kwh = +(inst._systemInfo.capacity * monitoringData.soc).toFixed(2)

            monitoringData.soh = +(response.data[13] / 10.0).toFixed(1)

            monitoringData.dc_voltage = +(response.data[14] / 10).toFixed(1)
            monitoringData.dc_current = +(response.buffer.readInt16BE(buff_pos + 30) / 10).toFixed(1)
            monitoringData.charge_current_limit = +(response.data[16] / 10).toFixed(1)
            monitoringData.discharge_current_limit = +(response.data[17] / 10).toFixed(1)
            monitoringData.charge_power_limit = +(response.data[18] / 10).toFixed(1)
            monitoringData.discharge_current_limit = +(response.data[19] / 10).toFixed(1)
            monitoringData.max_cell_voltage = +(response.data[20] / 10).toFixed(1)
            monitoringData.max_cell_voltage_loc = response.data[21]
            monitoringData.min_cell_voltage = +(response.data[22] / 10).toFixed(1)
            monitoringData.min_cell_voltage_loc = response.data[23]
            monitoringData.max_module_temp = +(response.buffer.readInt16BE(buff_pos + 48) / 10).toFixed(1)
            monitoringData.max_module_temp_loc = response.data[25]
            monitoringData.min_module_temp = +(response.buffer.readInt16BE(buff_pos + 52) / 10).toFixed(1)
            monitoringData.min_module_temp_loc = response.data[27]

            console.log('monitoringData =', JSON.stringify(monitoringData, null, 4))
        }
    } catch (e) {
        console.error('await read error =', e)
    }

    close()
    // client.readInputRegisters(0, 56)
    //     .then(function(response) {
    //         console.log(response)
    //     })
    //     .catch(function(e) {
    //         console.log(e.message);
    //     })
    //     .then(close);
}

function close() {
    client.close()
}