/* eslint-disable no-console, spaced-comment */

// create an empty modbus client
//var ModbusRTU = require("modbus-serial");
var ModbusRTU = require("../index");
var client = new ModbusRTU();

// open connection to a serial port
//client.connectRTU("/dev/ttyUSB0", {baudRate: 9600})
client.connectTcpDirectRTUBuffer("sinan-ecs.nbplus.kr", {
        port: 10502
    })
    .then(setClient)
    .then(function() {
        console.log("Connected");
    })
    .catch(function(e) {
        console.log(e.message);
    });

function setClient() {
    // set the client's unit id
    // set a timout for requests default is null (no timeout)
    client.setID(1);
    client.setTimeout(1000);

    // run program
    run();
}

async function run() {
    client.setID(1)
    try {
        var response = await client.readInputRegisters(0, 56)
        if (response) {
            console.log('Received data =', response)

            var monitoringData = {}
            // parse data, save database and push json
            //logger.debug(TAG, 'response =', response)
            // node_module에서 data 부분을 readUInt16BE로 변환한다.
            // signed, unsigned가 섞여 있으므로 signed는 buffer에서 직접 읽어서 처리한다.

            // 4 바이트씩이다.
            // 각각의 전력량(Energy) 계산 방법:: 최종값 = High + Low / 10,000
            var buff_pos = 12
            monitoringData.pv_vr = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_vs = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_vt = response.buffer.readFloatBE(buff_pos)

            buff_pos += 20
            monitoringData.pv_max_act_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_cr = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_cs = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_ct = response.buffer.readFloatBE(buff_pos)

            buff_pos += 12
            monitoringData.pv_max_rev_act_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 8
            monitoringData.pv_freq = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_max_rev_react_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_act_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_react_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_rev_act_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_rev_react_power = response.buffer.readFloatBE(buff_pos)

            buff_pos += 8
            monitoringData.pv_act_energy = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_react_energy = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_rev_act_energy = response.buffer.readFloatBE(buff_pos)

            console.log('monitoringData =', JSON.stringify(monitoringData, null, 4))
        }

        client.setID(2)
        var response = await client.readInputRegisters(0, 44)
        if (response) {
            console.log('Received data =', response)
            var monitoringData = {}
            // parse data, save database and push json
            //logger.debug(TAG, 'response =', response)
            // node_module에서 data 부분을 readUInt16BE로 변환한다.
            // signed, unsigned가 섞여 있으므로 signed는 buffer에서 직접 읽어서 처리한다.

            // 4 바이트씩이다.
            // 각각의 전력량(Energy) 계산 방법:: 최종값 = High + Low / 10,000
            var buff_pos = 0
            monitoringData.pv_gmac_avg_v = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_avg_c = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_vr = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_vs = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_vt = response.buffer.readFloatBE(buff_pos)

            buff_pos += 28
            monitoringData.pv_gmac_power_factor = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_total_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_total_react_power = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_total_apparent_power = response.buffer.readFloatBE(buff_pos)

            buff_pos += 4
            monitoringData.pv_gmac_freq = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_act_energy = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_react_energy = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.pv_gmac_apparent_energy = response.buffer.readFloatBE(buff_pos)

            console.log('monitoringData =', JSON.stringify(monitoringData, null, 4))
        }
    } catch (e) {
        console.error('await read error =', e)
    }

    close()
    // client.readInputRegisters(0, 56)
    //     .then(function(response) {
    //         console.log(response)
    //     })
    //     .catch(function(e) {
    //         console.log(e.message);
    //     })
    //     .then(close);
}

function close() {
    client.close()
}