/* eslint-disable no-console, spaced-comment */

// create an empty modbus client
//var ModbusRTU = require("modbus-serial");
var ModbusRTU = require("../index");
var client = new ModbusRTU();
var moment = require('moment')

var pcs_error_codes = [{
        code: 'PCS_DESTIN_00000001',
        level: 0x00,
        desc: "PCS Start Failure"
    },
    {
        code: 'PCS_DESTIN_00000002',
        level: 0x00,
        desc: "Batt Over Voltage"
    },
    {
        code: 'PCS_DESTIN_00000004',
        level: 0x00,
        desc: "Batt Over Current"
    },
    {
        code: 'PCS_DESTIN_00000008',
        level: 0x00,
        desc: "Batt Under Voltage"
    },
    {
        code: 'PCS_DESTIN_00000010',
        level: 0x00,
        desc: "DC CB1 Trip"
    },
    {
        code: 'PCS_DESTIN_00000020',
        level: 0x00,
        desc: "DC CB1A Fault"
    },
    {
        code: 'PCS_DESTIN_00000040',
        level: 0x00,
        desc: "DC Polarity Reversal"
    },
    null,
    {
        code: 'PCS_DESTIN_00000100',
        level: 0x00,
        desc: "DC Side Abnormal"
    },
    {
        code: 'PCS_DESTIN_00000200',
        level: 0x00,
        desc: "Inv. Over Voltage"
    },
    {
        code: 'PCS_DESTIN_00000400',
        level: 0x00,
        desc: "Inv. Under Voltage"
    },
    {
        code: 'PCS_DESTIN_00000800',
        level: 0x00,
        desc: "Inv. Over Frequency"
    },
    {
        code: 'PCS_DESTIN_00001000',
        level: 0x00,
        desc: "Inv. Under Frequency"
    },
    {
        code: 'PCS_DESTIN_00002000',
        level: 0x00,
        desc: "Inv. Over Current"
    },
    {
        code: 'PCS_DESTIN_00004000',
        level: 0x00,
        desc: "Inv. Phase Order Failure"
    },
    null,
    {
        code: 'PCS_DESTIN_00010001',
        level: 0x00,
        desc: "Inv. Current Unbalance"
    },
    {
        code: 'PCS_DESTIN_00010002',
        level: 0x00,
        desc: "Battery Current Sensor Abnormal"
    },
    {
        code: 'PCS_DESTIN_00010004',
        level: 0x00,
        desc: "PEBB Over Current"
    },
    {
        code: 'PCS_DESTIN_00010008',
        level: 0x00,
        desc: "PEBB Over Temp."
    },
    {
        code: 'PCS_DESTIN_00010010',
        level: 0x00,
        desc: "PEBB FAN Fault"
    },
    {
        code: 'PCS_DESTIN_00010020',
        level: 0x00,
        desc: "PEBB Fuse Fault"
    },
    {
        code: 'PCS_DESTIN_00010040',
        level: 0x00,
        desc: "PEBB IGBT Fault"
    },
    {
        code: 'PCS_DESTIN_00010080',
        level: 0x00,
        desc: "PEBB MC Fault"
    },
    null, null, null, null,
    {
        code: 'PCS_DESTIN_00011000',
        level: 0x00,
        desc: "CB2 Fault"
    },
    {
        code: 'PCS_DESTIN_00012000',
        level: 0x00,
        desc: "Grid Over Voltage Level1"
    },
    {
        code: 'PCS_DESTIN_00014000',
        level: 0x00,
        desc: "Grid Under Voltage Level1"
    },
    {
        code: 'PCS_DESTIN_00018000',
        level: 0x00,
        desc: "Grid Over Frequency Level1"
    },
    {
        code: 'PCS_DESTIN_00020001',
        level: 0x00,
        desc: "Grid Under Frequency Level1"
    },
    {
        code: 'PCS_DESTIN_00020002',
        level: 0x00,
        desc: "Grid CB3 Fault"
    },
    null,
    null,
    null,
    null,
    null,
    {
        code: 'PCS_DESTIN_00020080',
        level: 0x00,
        desc: "MC4 Fault"
    },
    null,
    null,
    {
        code: 'PCS_DESTIN_00020400',
        level: 0x00,
        desc: "TR Over Temp"
    },
    null,
    {
        code: 'PCS_DESTIN_00021000',
        level: 0x00,
        desc: "NVSRAM Failure"
    },
    {
        code: 'PCS_DESTIN_00022000',
        level: 0x00,
        desc: "ASYNC Failure"
    },
    {
        code: 'PCS_DESTIN_00024000',
        level: 0x00,
        desc: "CAN Failure"
    },
    null,
    {
        code: 'PCS_DESTIN_00040001',
        level: 0x00,
        desc: "Emergency Stop"
    },
    null,
    {
        code: 'PCS_DESTIN_00040004',
        level: 0x00,
        desc: "Ground Fault Detection (DC)"
    },
    {
        code: 'PCS_DESTIN_00040008',
        level: 0x00,
        desc: "Parameter Error"
    },
    {
        code: 'PCS_DESTIN_00040010',
        level: 0x00,
        desc: "PCS 판넬 도어 개방"
    },
    null,
    null,
    {
        code: 'PCS_DESTIN_00040080',
        level: 0x00,
        desc: "Ground Fault Detection (AC)"
    },
    null,
    null,
    null,
    {
        code: 'PCS_DESTIN_00040800',
        level: 0x00,
        desc: "PEBB Temp Unbalance"
    },
    null,
    null,
    null,
    null,
    null,
    {
        code: "PCS_DESTIN_00100002",
        level: 0x03,
        desc: "SOC Warning"
    },
    {
        code: "PCS_DESTIN_00100004",
        level: 0x03,
        desc: "Idle Warning"
    },
    null, null, null, null, null, null, null, null, null,
    {
        code: "PCS_DESTIN_00101000",
        level: 0x03,
        desc: "Temperature Derease mode ON"
    },
    {
        code: "PCS_DESTIN_00102000",
        level: 0x03,
        desc: "GFD Warning"
    },
    {
        code: "PCS_DESTIN_00104000",
        level: 0x03,
        desc: "MC5 Warning"
    },

    {
        code: "PCS_DESTIN_00108000",
        level: 0x03,
        desc: "SPD"
    },
    {
        code: "PCS_DESTIN_00200001",
        level: 0x03,
        desc: "Backfeed Protection"
    },
    null, null, null, null, null, null, null, null, null, null, null, null, null,
    {
        code: "PCS_DESTIN_00204000",
        level: 0x03,
        desc: "PEBB-eGBI Parameter Init"
    },
    null, null, null, null, null, null, null, null, null,
    {
        code: "PCS_DESTIN_00400100",
        level: 0x03,
        desc: "Test Mode"
    },
    null, null, null, null,
    {
        code: "PCS_DESTIN_00402000",
        level: 0x03,
        desc: "Invalid Parameter"
    },
    null, null, null, null, null, null, null, null,
    {
        code: "PCS_DESTIN_00800040",
        level: 0x03,
        desc: "SMPS AC"
    },
    {
        code: "PCS_DESTIN_00800080",
        level: 0x03,
        desc: "SMPS DC"
    },
    null, null, null, null, null, null,
    {
        code: "PCS_DESTIN_00804000",
        level: 0x03,
        desc: "AC DOOR OPEN"
    },
    {
        code: "PCS_DESTIN_00808000",
        level: 0x03,
        desc: "DC DOOR OPEN"
    }
]
var pcs_protection_status_length = 64

var pcs_pebb_error_codes = [{
        code: "PCS_DESTIN_01000001",
        level: 0x01,
        desc: "PEBB : Over Current"
    },
    null, null, null, null,
    {
        code: "PCS_DESTIN_01000020",
        level: 0x01,
        desc: "PEBB : IGBT R Fault"
    },

    {
        code: "PCS_DESTIN_01000040",
        level: 0x01,
        desc: "PEBB : IGBT S Fault"
    },
    {
        code: "PCS_DESTIN_01000080",
        level: 0x01,
        desc: "PEBB : IGBT T Fault"
    },
    {
        code: "PCS_DESTIN_01000100",
        level: 0x01,
        desc: "PEBB : IGBT Over Temp"
    },
    {
        code: "PCS_DESTIN_01000200",
        level: 0x01,
        desc: "PEBB : FAN Fault"
    },
    {
        code: "PCS_DESTIN_01000400",
        level: 0x01,
        desc: "PEBB : FUSE Fault"
    },
    {
        code: "PCS_DESTIN_01000800",
        level: 0x01,
        desc: "PEBB : MC Fault"
    },
    null,
    {
        code: "PCS_DESTIN_01002000",
        level: 0x01,
        desc: "PEBB : Heatsink Over Temp"
    },
    null, null
]

// open connection to a serial port
//client.connectRTU("/dev/ttyUSB0", {baudRate: 9600})

connectClient()

function connectClient() {
    client.connectTCP("sinan-ecs.nbplus.kr", {
            port: 12502
        })
        .then(setClient)
        .then(function() {
            console.log("Connected");
        })
        .catch(function(e) {
            console.log(e.message);
        });
}

var timer

function setClient() {
    // set the client's unit id
    // set a timout for requests default is null (no timeout)
    client.setID(1);
    client.setTimeout(1000);

    // run program
    timer = setInterval(run, 1000)
    client._port.on('close', function(key) {
        console.log('closed............ reconnect key =', key)
        connectClient()
        if (timer) {
            clearInterval(timer)
        }
    })
    //run();
}

async function run() {
    client.setID(1)
    try {
        var response = await client.readHoldingRegisters(20, 66)
        if (response) {
            console.log('Received data =', response)

            var monitoringData = {}

            var buff_pos = 0
            monitoringData.batt_v = +(response.buffer.readInt16BE(buff_pos) / 10.0).toFixed(1)
            monitoringData.dc_a = +(response.buffer.readInt16BE(buff_pos + 2) / 10.0).toFixed(1)
            monitoringData.dc_v = +(response.buffer.readInt16BE(buff_pos + 4) / 10.0).toFixed(1)
            monitoringData.dc_p = +(response.buffer.readInt16BE(buff_pos + 6) / 10.0).toFixed(1)

            // 29
            monitoringData.pcs_vr = +(response.data[9] / 10).toFixed(1)
            monitoringData.pcs_vs = +(response.data[10] / 10).toFixed(1)
            monitoringData.pcs_vt = +(response.data[11] / 10).toFixed(1)
            monitoringData.pcs_ar = +(response.data[12] / 10).toFixed(1)
            monitoringData.pcs_as = +(response.data[13] / 10).toFixed(1)
            monitoringData.pcs_at = +(response.data[14] / 10).toFixed(1)
            monitoringData.pcs_freq = +(response.data[15] / 10).toFixed(1)

            // 37
            monitoringData.grid_vr = +(response.data[17] / 10).toFixed(1)
            monitoringData.grid_vs = +(response.data[18] / 10).toFixed(1)
            monitoringData.grid_vt = +(response.data[19] / 10).toFixed(1)
            monitoringData.grid_ar = +(response.data[20] / 10).toFixed(1)
            monitoringData.grid_as = +(response.data[21] / 10).toFixed(1)
            monitoringData.grid_at = +(response.data[22] / 10).toFixed(1)
            monitoringData.grid_freq = +(response.data[24] / 10).toFixed(1)

            // 45
            monitoringData.power_p = +(response.buffer.readInt16BE(buff_pos + 50) / 10).toFixed(1)
            monitoringData.power_q = +(response.buffer.readInt16BE(buff_pos + 52) / 10).toFixed(1)
            monitoringData.power_factor = +(response.data[27] / 1000).toFixed(3)
            monitoringData.order_source = response.data[28]
            // 4byte
            monitoringData.pcs_status = response.buffer.readUInt32BE(buff_pos + 58)
            monitoringData.pcs_status_obj = {}
            var pcs_status = response.buffer.readInt8(buff_pos + 58)
            monitoringData.pcs_status_obj.heavy_fault = (pcs_status & 0x01) >> 1
            monitoringData.pcs_status_obj.cb1_onoff = (pcs_status & 0x02) >> 2
            monitoringData.pcs_status_obj.cb1a_onoff = (pcs_status & 0x04) >> 3
            monitoringData.pcs_status_obj.mc4_onoff = (pcs_status & 0x02) >> 5
            monitoringData.pcs_status_obj.cb3_onoff = (pcs_status & 0x04) >> 6

            pcs_status = response.buffer.readInt8(buff_pos + 59)
            monitoringData.pcs_status_obj.batt_fault = (pcs_status & 0x02) >> 1
            monitoringData.pcs_status_obj.pcs_fault = (pcs_status & 0x04) >> 2
            monitoringData.pcs_status_obj.grid_fault = (pcs_status & 0x08) >> 3

            pcs_status = response.buffer.readInt8(buff_pos + 61)
            monitoringData.pcs_status_obj.system_command_onoff = (pcs_status & 0x02) >> 7

            // 51
            monitoringData.charge_status = response.data[31]
            monitoringData.pebb_mc_status = response.data[32]
            monitoringData.control_mode_state = response.data[33]
            monitoringData.control_mode_state_obj = {}
            monitoringData.control_mode_state_obj.state = (response.data[33] & 0xF0) >> 4
            monitoringData.control_mode_state_obj.mode = response.data[33] & 0x0F
            monitoringData.pcs_fault = Buffer.allocUnsafe(16)
            monitoringData.pcs_fault.fill(0)
            response.buffer.copy(monitoringData.pcs_fault, buff_pos + 68, 16)

            var errbyte
            var byte_pos = buff_pos + 68
            var op = 0x01

            var date = moment().format('YYYY-MM-DD HH:mm:ss')

            monitoringData.protection_status = {}
            for (var i = 0; i < pcs_error_codes.length; i++) {
                if (isNullObject(pcs_error_codes[i])) {
                    continue
                }
                var bit_pos = i % 8
                if (bit_pos == 0) {
                    errbyte = response.buffer.readInt8(byte_pos + (i / 8))
                }

                var isError = (((errbyte >> bit_pos) & op) == op)
                // PCS_DESTIN_00000001: {
                //     desc: 'PCS Start Failure',
                //     err_yn: false
                // },
                if (i < pcs_protection_status_length) {
                    monitoringData.protection_status[pcs_error_codes[i].code] = {
                        desc: pcs_error_codes[i].desc,
                        err_yn: isError
                    }
                }
                if (!isError) {
                    continue
                }
                // ECS_00000002: {
                //     level: 0x00,
                //     desc: 'PCS 통신 장애',
                //     date: date
                // }
                error.events[pcs_error_codes[i].code] = {
                    level: pcs_error_codes[i].level,
                    desc: pcs_error_codes[i].desc,
                    num_rack: 0,
                    date: date
                }
            }

            monitoringData.pebb_fault = Buffer.allocUnsafe(16)
            monitoringData.pebb_fault.fill(0)
            response.buffer.copy(monitoringData.pebb_fault, buff_pos + 84, 16)
            byte_pos = buff_pos + 84
            for (var i = 0; i < pcs_pebb_error_codes.length; i++) {
                if (isNullObject(pcs_pebb_error_codes[i])) {
                    continue
                }
                var bit_pos = i % 8
                var pebb_pos = (i / 8) + 1
                if (bit_pos == 0) {
                    errbyte = response.buffer.readInt8(byte_pos + (i / 8))
                }

                var isError = (((errbyte >> bit_pos) & op) == op)
                if (!isError) {
                    continue
                }
                // ECS_00000002: {
                //     level: 0x00,
                //     desc: 'PCS 통신 장애',
                //     date: date
                // }
                error.events[pcs_pebb_error_codes[i].code] = {
                    level: pcs_pebb_error_codes[i].level,
                    desc: pcs_pebb_error_codes[i].desc,
                    num_rack: pebb_pos,
                    date: data
                }
            }

            //ecserrors.setErrors(error)
            // 70
            monitoringData.control_state = response.data[50]
            monitoringData.cabinet_temp = response.data[51]

            // pebb_heatsync_temp
            var pebbtempArr = []
            for (var i = 0; i < 8; i++) {
                pebbtempArr[i] = response.data[52 + i]
            }
            monitoringData.pebb_heatsync_temp = pebbtempArr
            //monitoringData.pebb_heatsync_temp_buffer = Buffer.allocUnsafe(8)
            //response.buffer.copy(monitoringData.pebb_heatsync_temp_buffer, buff_pos + 104, 16)

            // 84 ~ 85
            monitoringData.cpmode_powerp_ref = +(response.buffer.readInt16BE(buff_pos + 128) / 10).toFixed(1)
            monitoringData.cpmode_powerq_ref = +(response.buffer.readInt16BE(buff_pos + 130) / 10).toFixed(1)

            console.log('monitoringData =', JSON.stringify(monitoringData, null, 4))
        }
    } catch (e) {
        console.error('await read error =', e)
    }

    //console.log('close...')
    //close()
    // client.readInputRegisters(0, 56)
    //     .then(function(response) {
    //         console.log(response)
    //     })
    //     .catch(function(e) {
    //         console.log(e.message);
    //     })
    //     .then(close);
}

function close() {
    client.close()
}