/* eslint-disable no-console, spaced-comment */

// create an empty modbus client
//var ModbusRTU = require("modbus-serial");
var ModbusRTU = require("../index");
var client = new ModbusRTU();

// open connection to a serial port
//client.connectRTU("/dev/ttyUSB0", {baudRate: 9600})
client.connectTCP("sinan-ecs.nbplus.kr", {
        port: 11502
    })
    .then(setClient)
    .then(function() {
        console.log("Connected");
    })
    .catch(function(e) {
        console.log(e.message);
    });

function setClient() {
    // set the client's unit id
    // set a timout for requests default is null (no timeout)
    client.setID(1);
    client.setTimeout(1000);

    // run program
    run();
}

async function run() {
    client.setID(1)
    try {
        var response = await client.readInputRegisters(0, 72)
        if (response) {
            console.log('Received data =', response)

            var monitoringData = {}
            // parse data, save database and push json
            //logger.debug(TAG, 'response =', response)
            // node_module에서 data 부분을 readUInt16BE로 변환한다.
            // signed, unsigned가 섞여 있으므로 signed는 buffer에서 직접 읽어서 처리한다.

            // 4 바이트씩이다.
            // 각각의 전력량(Energy) 계산 방법:: 최종값 = High + Low / 10,000
            var buff_pos = 0
            monitoringData.ess_vr = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.ess_vs = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.ess_vt = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.ess_vn = response.buffer.readFloatBE(buff_pos)

            buff_pos += 20
            monitoringData.ess_cr = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.ess_cs = response.buffer.readFloatBE(buff_pos)
            buff_pos += 4
            monitoringData.ess_ct = response.buffer.readFloatBE(buff_pos)

            buff_pos += 16
            monitoringData.ess_freq = response.buffer.readFloatBE(buff_pos)

            buff_pos += 12
            monitoringData.ess_act_power = +(response.buffer.readFloatBE(buff_pos) / 1000).toFixed(3)
            buff_pos += 4
            monitoringData.ess_react_power = +(response.buffer.readFloatBE(buff_pos) / 1000).toFixed(3)

            // 전력량 읽기 (*1000은 kWh 또는 kVarH)
            buff_pos += 8
            monitoringData.ess_act_energy = +((response.buffer.readInt32BE(buff_pos) + (response.buffer.readInt32BE(buff_pos + 4) / 10000)) * 1000).toFixed(3)
            //TODO : 양수가 충전인지 방전인지 ????
            if (monitoringData.ess_act_energy >= 0) {
                monitoringData.ess_act_energy_charge = monitoringData.ess_act_energy
                monitoringData.ess_act_energy_discharge = 0
            } else {
                monitoringData.ess_act_energy_charge = 0
                monitoringData.ess_act_energy_discharge = monitoringData.ess_act_energy * -1
            }

            buff_pos += 8
            monitoringData.ess_react_energy_lead = +((response.buffer.readInt32BE(buff_pos) + (response.buffer.readInt32BE(buff_pos + 4) / 10000)) / 1000).toFixed(3)
            buff_pos += 8
            monitoringData.ess_react_energy_lag = +((response.buffer.readInt32BE(buff_pos) + (response.buffer.readInt32BE(buff_pos + 4) / 10000)) / 1000).toFixed(3)
            buff_pos += 16
            monitoringData.ess_rev_act_energy = +((response.buffer.readInt32BE(buff_pos) + (response.buffer.readInt32BE(buff_pos + 4) / 10000)) / 1000).toFixed(3)
            buff_pos += 8
            monitoringData.ess_rev_react_energy_lead = +((response.buffer.readInt32BE(buff_pos) + (response.buffer.readInt32BE(buff_pos + 4) / 10000)) / 1000).toFixed(3)
            buff_pos += 8
            monitoringData.ess_rev_react_energy_lag = +((response.buffer.readInt32BE(buff_pos) + (response.buffer.readInt32BE(buff_pos + 4) / 10000)) / 1000).toFixed(3)

            console.log('monitoringData =', JSON.stringify(monitoringData, null, 4))
        }
    } catch (e) {
        console.error('await read error =', e)
    }

    close()
    // client.readInputRegisters(0, 56)
    //     .then(function(response) {
    //         console.log(response)
    //     })
    //     .catch(function(e) {
    //         console.log(e.message);
    //     })
    //     .then(close);
}

function close() {
    client.close()
}